/* 	
	Config file parser

	Copyright Jeroen Vreeken (pe1rxq@amsat.org), 2007
	Copyright Stichting C.A. Muller Radioastronomiestation, 2007

	This program is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

void config_string(void *arg, void *dst)
{
	char **dststr = dst;
	
	*dststr = strdup(arg);
}

void config_double(void *arg, void *dst)
{
	*(double *)dst = atof(arg);
}

void config_int(void *arg, void *dst)
{
	*(int *)dst = atoi(arg);
}

struct config_option {
	char *opt;
	void (*parser)(void *, void *);
	void *dst;
	struct config_option *next;
};

static struct config_option *option_list = NULL;

void config_add(char *opt, void (*parser)(void *, void *), void *dst)
{
	struct config_option *option;
	
	option = malloc(sizeof(struct config_option));
	option->opt = opt;
	option->parser = parser;
	option->next = option_list;
	option->dst = dst;
	option_list = option;
}

void config_read(char *file)
{
	FILE *fd;
	char buffer[256];
	
	fd = fopen(file, "r");
	if (!fd)
		return;
	
	while (fgets(buffer, 256, fd) != NULL) {
		char *opt;
		char *val;
		char *ptr;
		struct config_option *entry;
		
		if (buffer[0] == '#' ||
		    buffer[0] == ';')
			continue;
		
		opt = strtok_r(buffer, " =:\t\n\r", &ptr);
		val = strtok_r(NULL, " \t\n\r", &ptr);
		
		if (!opt || ! val)
			continue;
		for (entry = option_list; entry; entry = entry->next)
			if (strcmp(entry->opt, opt) == 0)
				entry->parser(val, entry->dst);
	}

	fclose(fd);
}
