/*
        weather.c

        Copyright Jeroen Vreeken (pe1rxq@amsat.org), 2009
        Copyright Stichting C.A. Muller Radioastronomiestation, 2009

        This program is free software: you can redistribute it and/or modify
        it under the terms of the GNU General Public License as published by
        the Free Software Foundation, either version 3 of the License, or
        (at your option) any later version.

        This program is distributed in the hope that it will be useful,
        but WITHOUT ANY WARRANTY; without even the implied warranty of
        MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
        GNU General Public License for more details.

        You should have received a copy of the GNU General Public License
        along with this program.  If not, see <http://www.gnu.org/licenses/>.

 */


#include "weather.h"

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <unistd.h>
#include <pthread.h>
#include <limits.h>
#include "tcp_connect.h"

struct weather {
	double temperature;
	double pressure;
};

#define WEATHER_HTTP_INTERVAL	60
#define WEATHER_HTTP_URL	"http://www.knmi.nl/actueel/index.html"
#define WEATHER_HTTP_HOST	"www.knmi.nl"

#define WEATHER_HTTP_REQUEST	"GET http://www.knmi.nl/actueel/index.html HTTP/1.0\nUser-Agent: CAMRAS-weather/0.1\n\n"

#define WEATHER_HTTP_LOCATION	"Hoogeveen"

static char weather_http_reply[1024*1024];

static void *weather_http(void *arg)
{
	struct weather *weather = arg;
	do {
		int fd;
		
		fd = tcp_connect(WEATHER_HTTP_HOST, 80);
		
		if (fd >= 0) {
			int ret, i;
			int inputsize = 0;
			char *reply = weather_http_reply;
			char *req = WEATHER_HTTP_REQUEST;

			write(fd, req, strlen(req));
			do {
			 	ret = read(fd, &reply[inputsize], 1);
				inputsize++;
			} while (ret == 1);
			
			close(fd);
			reply[inputsize] = 0;
			for (i = 0; i < inputsize; i++) {
				if (!strncmp(&reply[i],
				    WEATHER_HTTP_LOCATION,
				    strlen(WEATHER_HTTP_LOCATION))) {
					char *tok;
					double temp = -270, pres = 0;
					
					tok = strtok(&reply[i], "<>");
					if (tok) tok = strtok(NULL, "</>");
					if (tok) tok = strtok(NULL, "</>");
					if (tok) tok = strtok(NULL, "</>");
					if (tok) tok = strtok(NULL, "</>");
					if (tok) tok = strtok(NULL, "</>");
					if (tok) tok = strtok(NULL, "</>");
					if (tok) tok = strtok(NULL, "</>");
					if (tok) tok = strtok(NULL, "</&>");
					if (tok) {
						temp = atof(tok);
					}
					if (tok) tok = strtok(NULL, ">");
					if (tok) tok = strtok(NULL, ">");
					if (tok) tok = strtok(NULL, ">");
					if (tok) tok = strtok(NULL, ">");
					if (tok) tok = strtok(NULL, ">");
					if (tok) tok = strtok(NULL, ">");
					if (tok) tok = strtok(NULL, ">");
					if (tok) tok = strtok(NULL, ">");
					if (tok) tok = strtok(NULL, ">");
					if (tok) tok = strtok(NULL, ">");
					if (tok) tok = strtok(NULL, ">&;<");
					if (tok) {
						pres = atof(tok);
					}
					if (pres > 900 &&
					    pres < 1100 &&
					    temp > -40 &&
					    temp < 50) {
						weather->temperature = temp;
						weather->pressure = pres;
					}

					break;
				}
			}
		}
	
		sleep(WEATHER_HTTP_INTERVAL);
	} while (1);
	
	return NULL;
}

struct weather *weather_create(char *location)
{
	struct weather *weather;
	pthread_t thread_id;
	pthread_attr_t attr;
	
	if (strcmp(location, "Dwingeloo")) {
		printf("weather: Only the location 'Dwingeloo' is supported\n");
	}
	
	weather = malloc(sizeof(struct weather));
	if (weather) {
		weather->temperature = 11;
		weather->pressure = 1013;
	}
	
	pthread_attr_init(&attr);
	pthread_attr_setstacksize(&attr, PTHREAD_STACK_MIN * 2);
	
	pthread_create(&thread_id, &attr, weather_http, weather);
	
	return weather;
}

double weather_get_temperature(struct weather *weather)
{
	return weather->temperature;
}

double weather_get_pressure(struct weather *weather)
{
	return weather->pressure;
}
