/* 
	Javascript with DT UI elements

	Copyright Jeroen Vreeken (pe1rxq@amsat.org), 2011, 2014
	Copyright Stichting C.A. Muller Radioastronomiestation, 2011

	This program is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program.  If not, see <http://www.gnu.org/licenses/>.

 */


var dt_ui_loaded;
if (!dt_ui_loaded) {
dt_ui_loaded = true;

dt_ui_color_text = "black";
dt_ui_color_text_title = "blue";
dt_ui_color_background = "lightgrey";
dt_ui_color_background_value = "white";
dt_ui_color_background_title = "silver";
dt_ui_color_indicator_off = "grey";
dt_ui_color_indicator_green = "#00ff00";
dt_ui_color_indicator_red = "#ff0000";

dt_ui_padding_height_window = 10;
dt_ui_padding_width_window = 10;

eval(load("trace.js"));

function strip_px(pos)
{
	return parseInt(pos.replace("px", "")) | 0;
}

function dt_ui_element(elementid)
{
	if (elementid === undefined)
		return;

	var create_element = (this.element === undefined);
	
	this.name = elementid;
	if (create_element)
		this.element = document.getElementById(elementid);
	if (this.element == null) {
		this.element = document.createElement("span");
		this.element.id = elementid;
	}
	this.element.style.display = "inline-block";
	this.style_display = "inline-block";
	this.element.style.cssFloat = "none";
	this.element.style.position = "absolute";
	this.element.style.left = "0px";
	this.element.style.top = "0px";

//debug
//	this.element.style.borderStyle = "solid";
//	this.element.style.borderWidth = "1px";
	
	this.padding_height = 0;
	this.padding_width = 0;
}
dt_ui_element.prototype.add = function (element)
{
	if (Array.isArray(element)) {
		var i;
		
		for (i = 0; i < element.length; i++) {
			this.add(element[i]);
		}
	} else {
		this.element.appendChild(element.element);
		element.parent = this;
	}
}
dt_ui_element.prototype.width_set = function(w)
{
	this.element.style.width = w + "px";
//	alert(this.name + ": width_set " + w);
}
dt_ui_element.prototype.width_get = function()
{
//	alert(this.name + ": width get: " + this.element.offsetWidth);
	return this.element.offsetWidth;
}
dt_ui_element.prototype.height_set = function(h)
{
	this.element.style.height = h + "px";
//	alert(this.name + ": height_set " + h);
}
dt_ui_element.prototype.height_get = function()
{
	return this.element.offsetHeight;
}
dt_ui_element.prototype.top_set = function(t)
{
	if (this.parent)
		t += this.parent.padding_height;
	this.element.style.top = t + "px";
//	alert(this.name + ": top_set " + t +" "+ this.parent.padding_height);
}
dt_ui_element.prototype.top_get = function()
{
	var t = this.element.offsetTop;
	if (this.parent)
		t -= this.parent.padding_height;
	if (t < 0)
		t = 0;
	return t;
}
dt_ui_element.prototype.left_set = function(l)
{
//	var org = l;
	if (this.parent)
		l += this.parent.padding_width;
	this.element.style.left= l + "px";
//	alert(this.name + ": left_set " + l + " " + org +"  "+this.parent.padding_width);
}
dt_ui_element.prototype.left_get = function()
{
	var l = this.element.offsetLeft; 
	if (this.parent)
		l -= this.parent.padding_height;
	if (l < 0)
		l = 0;
	return l;
}
dt_ui_element.prototype.resize_equal_width = function(elements)
{
	var i;
	var maxw = 0;
	
	for (i = 0; i < elements.length; i++) {
		maxw = Math.max(maxw, elements[i].element.offsetWidth);
	}
	for (i = 0; i < elements.length; i++) {
		elements[i].width_set(maxw);
	}
}
dt_ui_element.prototype.resize_equal_height = function(elements)
{
	var i;
	var maxh = 0;
	
	for (i = 0; i < elements.length; i++) {
		maxh = Math.max(maxh, elements[i].element.offsetHeight);
	}
	for (i = 0; i < elements.length; i++) {
		elements[i].height_set(maxh);
	}
}
dt_ui_element.prototype.resize_equal = function(elements)
{
	this.resize_equal_width(elements);
	this.resize_equal_height(elements);
}
dt_ui_element.prototype.room = function(element)
{
	var w;
	var h;
	
	w = element.left_get() + element.width_get();
	h = element.top_get() + element.height_get();
	w += this.padding_width * 2;
	h += this.padding_height * 2;
	
	if (w > this.width_get()) {
		this.width_set(w);
	}
	if (h > this.height_get()) {
		this.height_set(h);
	}
	if(this.parent)
		this.parent.room(this);
}
dt_ui_element.prototype.align_vertical = function(elements, spacing)
{
	var i;
	var left = elements[0].left_get();
	var top;

	spacing = typeof spacing == 'undefined' ? 0 : spacing;

	top = elements[0].top_get();
	for (i = 0; i < elements.length; i++) {
		elements[i].left_set(left);
		elements[i].top_set(top);
		this.room(elements[i]);
		top += elements[i].height_get();
		top += spacing;
	}
}
dt_ui_element.prototype.align_horizontal = function(elements, spacing)
{
	var i;
	var top = elements[0].top_get();
	var left;

	spacing = typeof spacing == 'undefined' ? 0 : spacing;

	left = elements[0].left_get();
	for (i = 0; i < elements.length; i++) {
		elements[i].left_set(left);
		elements[i].top_set(top);
//		alert(elements[i].name + ": " + left + "," + top);
		this.room(elements[i]);
		left += elements[i].width_get();
		left += spacing;
	}
}
dt_ui_element.prototype.background_set = function(color)
{
	this.element.style.background = color;
}
dt_ui_element.prototype.visible_set = function(visible)
{
	if (visible)
		this.element.style.display = this.style_display;
	else {
		this.element.style.display = "none";
	}
}
dt_ui_element.prototype.text_set = function (text)
{
	this.element.innerHTML = text;
	this.element.style.color = dt_ui_color_text;
	if (this.parent)
		this.parent.room(this);
}
dt_ui_element.prototype.wrap_set = function (wrap, pre)
{
	if (!wrap) {
		this.element.style.whiteSpace = "nowrap";
		return;
	}
	
	if (!pre){
		this.element.style.whiteSpace = "normal";
		return;
	}
	
	this.element.style.whiteSpace = "pre";
}


function dt_ui_canvas(elementid)
{
	this.element = document.createElement("canvas");
	this.element.id = elementid;
	dt_ui_element.call(this, elementid);
}
dt_ui_canvas.prototype = new dt_ui_element();


function dt_ui_title(elementid)
{
	dt_ui_element.call(this, elementid);
	this.title_set("untitled");
	this.element.style.whiteSpace = "nowrap";
	this.element.style.borderColor = "black";
	this.element.style.borderStyle = "solid";
	this.element.style.borderTopWidth = "0px";
	this.element.style.borderBottomWidth = "1px";
	this.element.style.borderLeftWidth = "0px";
	this.element.style.borderRightWidth = "1px";
	this.element.style.background = dt_ui_color_background_title;
}
dt_ui_title.prototype = new dt_ui_element();
dt_ui_title.prototype.title_set = function (titletext) {
	this.element.innerHTML = "<i>" + titletext + "</i>";
	this.element.style.color = dt_ui_color_text_title;
	if (this.parent)
		this.parent.room(this);
}


function dt_ui_window(elementid)
{
	if (elementid === undefined)
		return;

	dt_ui_element.call(this, elementid);

	this.title = new dt_ui_title(elementid + "_windowtitle");
	dt_ui_element.prototype.add.call(this, this.title);
		
	this.content = new dt_ui_element(elementid + "_windowcontent");
	this.content.element.style.position = "relative";
	dt_ui_element.prototype.add.call(this, this.content);
	this.content.top_set(this.title.height_get());
	
	this.content.padding_height = dt_ui_padding_height_window;
	this.content.padding_width = dt_ui_padding_width_window;

	this.element.style.border = "thin solid black";
	this.element.style.background = dt_ui_color_background;
	this.element.style.position = "relative";
}
dt_ui_window.prototype = new dt_ui_element();
dt_ui_window.prototype.title_set = function (titletext) 
{
	this.title.title_set(titletext);
}
dt_ui_window.prototype.add = function (element)
{
	this.content.add(element);
}
dt_ui_window.prototype.align_vertical = function(elements, spacing)
{
	this.content.align_vertical(elements, spacing);
}
dt_ui_window.prototype.align_horizontal = function(elements, spacing)
{
	this.content.align_horizontal(elements, spacing);
}

function dt_ui_window_hide(elementid)
{
	dt_ui_window.call(this, elementid);

	this.hide_ui = new dt_ui_button(elementid + "_windowhide");
	dt_ui_element.prototype.add.call(this, this.hide_ui);

	this.hide_ui.top_set(0);
	this.hide_ui.left_set(0);
	this.hide_ui.text_set("hide");
	
	var dt_ui_window_hide_this = this;
	
	var dt_ui_window_hide_visible = true;
	var dt_ui_window_hide_height;
	
	this.hide_ui.onclick = function(elem) {
		if (dt_ui_window_hide_visible) {
			dt_ui_window_hide_height =
			    dt_ui_window_hide_this.height_get();
			dt_ui_window_hide_this.height_set(
			    dt_ui_window_hide_this.title.height_get());
			dt_ui_window_hide_this.hide_ui.text_set("show");
		} else {
			dt_ui_window_hide_this.height_set(
			    dt_ui_window_hide_height);
			dt_ui_window_hide_this.hide_ui.text_set("hide");
		}

		dt_ui_window_hide_visible = !dt_ui_window_hide_visible;
		dt_ui_window_hide_this.content.visible_set(
		    dt_ui_window_hide_visible);
	}
}
dt_ui_window_hide.prototype = new dt_ui_window();
dt_ui_window_hide.prototype.title_set = function (titletext) 
{
	dt_ui_window.prototype.title_set.call(this, titletext);
	this.hide_ui.left_set(this.title.width_get());
}
dt_ui_window_hide.prototype.hide = function()
{
	this.hide_ui.onclick(null);
}


function dt_ui_value(elementid)
{
	dt_ui_element.call(this, elementid);

	this.align = "center";
	this.element.style.borderStyle = "solid";
	this.element.style.borderWidth = "1px";
	this.element.style.borderColor = "black";
	this.element.style.background = dt_ui_color_background_value;
	this.element.style.alignContent = "center";
	this.element.style.textAlign = this.align;
	this.element.style.display = "block";
	this.style_display = "block";
	this.element.innerHTML = "X";
	this.element.style.fontFamily = "monospace";
	this.element.style.fontSize = "120%";
}
dt_ui_value.prototype = new dt_ui_element();
dt_ui_value.prototype.value_set = function(value)
{
	this.element.innerHTML = value;
	if (this.parent)
		this.parent.room(this);
}
dt_ui_value.prototype.value_get = function(value)
{
	return this.element.innerHTML;
}


function dt_ui_select(elementid)
{
	this.element = document.createElement("select");
	dt_ui_element.call(this, elementid);

	this.element.onchange = function() {
		var opt = this.options[this.selectedIndex];
		
		if (opt.dt_ui_onclick)
			opt.dt_ui_onclick(opt.dt_ui_elem);
	}
}
dt_ui_select.prototype = new dt_ui_element();
dt_ui_select.prototype.option_add = function(name, onclick, elem)
{
	var option = document.createElement("option");
	
	option.text = name;
	option.onclick = function() {
		if (onclick)
			onclick(elem);
	}
	option.dt_ui_onclick = onclick;
	option.dt_ui_elem = elem;
	
	this.element.add(option, null);
}
dt_ui_select.prototype.option_clear = function()
{
	while (this.element.options.length)
		this.element.remove(0);
}


function dt_ui_text(elementid)
{
	this.element = document.createElement("input");
	this.element.type = "text";
	this.element.id = elementid;

	this.text_length;
	this.text_height = 1;

	dt_ui_element.call(this, elementid);
}
dt_ui_text.prototype = new dt_ui_element();
dt_ui_text.prototype.value_set = function(value)
{
	this.element.value = value;
	if (this.parent)
		this.parent.room(this);
}
dt_ui_text.prototype.value_get = function()
{
	return this.element.value;
}
dt_ui_text.prototype.value_set = function(val)
{
	this.element.value = val;
}
dt_ui_text.prototype.length_set = function(len)
{
	if (this.textheight == 1) {
		this.element.size = len;
	} else {
		this.element.cols = len;
	}
	this.text_length = len;
}


function dt_ui_textarea(elementid)
{
	this.element = document.createElement("textarea");
	this.element.id = elementid;

	this.text_length;

	dt_ui_element.call(this, elementid);
}
dt_ui_textarea.prototype = new dt_ui_element();
dt_ui_textarea.prototype.value_set = function(value)
{
	this.element.value = value;
	if (this.parent)
		this.parent.room(this);
}
dt_ui_textarea.prototype.value_get = function()
{
	return this.element.value;
}
dt_ui_textarea.prototype.value_set = function(val)
{
	this.element.value = val;
}
dt_ui_textarea.prototype.text_length_set = function(len)
{
	this.element.cols = len;
}
dt_ui_textarea.prototype.text_height_set = function(height)
{
	this.element.rows = height;
}



function dt_button(element)
{
	this.div = document.getElementById(element);
	
	this.button_element = 'dt_button_' + element;
	
	this.div.innerHTML = 
	    "<input type='button' id='" + this.button_element + "'>";
	this.button = document.getElementById(this.button_element);
	
	this.value = function dt_button_value(text) {
		this.button.value = text;
	}
	
	this.background = function dt_button_background(color)
	{
		this.button.style.background = color;
	}
	
	this.click = function dt_button_click(click_action)
	{
		this.button.onclick = click_action;
	}
}

var dt_button_border = "2px solid ";

function dt_button_indicator(element)
{
	this.div = document.getElementById(element);
	
	this.button_element = 'dt_button_' + element;
	this.button_ind_element = 'dt_button_ind_' + element;
	this.button_div_element = 'dt_button_div_' + element;
	
	this.div.style.borderColor = "lightgrey";
	this.div.style.borderStyle = "solid";
	this.div.style.borderTopWidth = "2px";
	this.div.style.borderBottomWidth = "4px";
	this.div.style.borderLeftWidth = "1px";
	this.div.style.borderRightWidth = "1px";
	this.div.innerHTML = 
	    "<button type='button' id='" + this.button_element + "'>" +
	    "<div style='" +
	    "display:inline-block; margin: 0px 4px -2px 0px; " +
	    "border:1px inset lightgrey; border-radius: 7px; width:12px; height:12px" +
	    "' id='" + this.button_ind_element + "'></div>" +
	    "<div style='display:inline-block' id='" + this.button_div_element + "'></div>" +
	    "</button>";
	this.button = document.getElementById(this.button_element);
	this.button_ind = document.getElementById(this.button_ind_element);
	this.button_div = document.getElementById(this.button_div_element);
	
	this.button_ind.style.background="black";
	
	this.value = function dt_button_value(text) {
		this.button_div.innerHTML = text;
	}
	
	this.background = function dt_button_background(color)
	{
		this.button_ind.style.background = color;
		this.div.style.borderColor = color;
	}
	this.indicator = this.background;
	
	this.click = function dt_button_click(click_action)
	{
		this.button.onclick = click_action;
	}
}

function dt_ui_button(elementid)
{
	this.element = document.createElement("button");
	this.element.id = elementid;

	dt_ui_element.call(this, elementid);

	this.button_text = new dt_ui_element(elementid + "_text");
	dt_ui_element.prototype.add.call(this, this.button_text);
//	this.button_text.element.style.borderStyle = "solid";
//	this.button_text.element.style.borderWidth = "2px";

	this.button_text.element.style.whiteSpace = "nowrap";

	this.padding_height = 4;
	this.padding_width = 4;

	var button_this = this;
	this.element.onclick = function()
	{
		if (button_this.onclick)
			button_this.onclick(button_this)
	}
}
dt_ui_button.prototype = new dt_ui_element();
dt_ui_button.prototype.text_set = function (text)
{
	this.button_text.element.innerHTML = text;
	this.button_text.top_set(0);
	this.button_text.left_set(0);
	this.room(this.button_text);
}
dt_ui_button.prototype.add = function (element)
{
	this.button_text.add(element);
	this.button_text.top_set(0);
	this.button_text.left_set(0);
	this.room(this.button_text);
}
dt_ui_button.prototype.color_set = function(color)
{
	this.element.style.background = color;
}
dt_ui_button.prototype.room = function (element)
{
	if (element == this.button_text)
		dt_ui_element.prototype.room.call(this, element);
	else
		this.button_text.room(element);
}

function dt_ui_indicator(elementid)
{
	dt_ui_element.call(this, elementid);

	this.element.style.border = "1px inset lightgrey";
	this.element.style.borderRadius = "7px";
	this.element.style.width = "12px";
	this.element.style.height = "12px";
	this.element.style.background = dt_ui_color_indicator_off;
}
dt_ui_indicator.prototype = new dt_ui_element();
dt_ui_indicator.prototype.color_set = function(color)
{
	this.element.style.background = color;
}

function dt_ui_button_indicator(elementid)
{
	dt_ui_button.call(this, elementid);

	this.indicator = new dt_ui_indicator(elementid + "_indicator");
	dt_ui_button.prototype.add.call(this, this.indicator);

	this.button_indicator_text = new dt_ui_element(elementid + "_ind_text");
	dt_ui_button.prototype.add.call(this, this.button_indicator_text);

//	this.button_indicator_text.element.style.borderStyle = "solid";
//	this.button_indicator_text.element.style.borderWidth = "2px";

//	this.element.style.borderStyle = "solid";
//	this.element.style.borderWidth = "2px"; 
//	this.element.style.borderColor = 
//	    dt_ui_color_indicator_off;
	
	this.text_set("?");
}
dt_ui_button_indicator.prototype = new dt_ui_button();
dt_ui_button_indicator.prototype.text_set = function (text)
{
	this.button_indicator_text.element.innerHTML = text;
	this.align_horizontal([this.indicator, this.button_indicator_text], 5);
}
dt_ui_button_indicator.prototype.add = function (element)
{
	this.button_indicator_text.add(element);
}
dt_ui_button_indicator.prototype.color_set = function(color)
{
	this.indicator.color_set(color);
//	this.element.style.borderColor = color;
}


function dt_traceval(element)
{
	var dt_traceval_this = this;
	
	this.div = document.getElementById(element);
	
	this.traceval_element = 'dt_traceval_' + element;

	this.label = '';
	this.suffix = '';
	this.align = 'center';

	this.align_set = function dt_traceval_align_set(alignment)
	{
		this.align = alignment;
		this.div_set();
	}
	
	this.div_set = function dt_traceval_div_set()
	{
		this.div.innerHTML = 
		    "<div "+
		    "style='" +
		    "background: lightgrey; " +
		    "border:thin solid black; " +
		    "text-align: " + this.align + "; " +
		    "font-family: monospace; font-size:120% ;" +
		    "white-space: pre " +
		    "' " +
		    "id='" + this.traceval_element + "' "+
		    ">" + this.label + "------" +
		    "</div>";
		this.traceval = document.getElementById(this.traceval_element);
	}
	this.div_set();

	this.previous_value = '------';
	
	this.fixed_number = false;
	this.fixed_number_decimals = 2;
	this.fixed_number_decimals_power = 100;

	this.highlight_enabled = false;
	this.highlight_color_high = 'red';
	this.highlight_color_normal = 'lightgrey';
	this.highlight_high = false;

	this.value_func = function (newval) {
		return newval;
	}

	this.value_func_set = function (func) {
		this.value_func = func;
	}

	this.value = function dt_traceval_value(new_value) {
		var changed = false;
		
		new_value = this.value_func(new_value);
		
		if (this.fixed_number) {
			new_value = Math.round(
			    this.fixed_number_decimals_power * new_value) /
			    this.fixed_number_decimals_power;
		}
	
		if (new_value != this.previous_value) {
			var filler = '';
			this.previous_value = new_value;
			
			if (this.fixed_number) {
				new_value = new_value.toFixed(this.fixed_number_decimals);
				if (new_value >= 0)
					filler = ' ';
			}
					
			dt_traceval_this.traceval.innerHTML =
			    "<div style='float: left'>" +
			    this.label + filler + 
			    "</div>" +
			    "<div align='right' style='padding-right: 5px' >" +
			    new_value +  this.suffix +
			    "</div>";
			changed = true;
			
			if (this.highlight_enabled && !this.highlight_high) {
				this.background(this.highlight_color_high);
				this.highlight_high = true;
			}
		} else if (this.highlight_enabled && this.highlight_high) {
			this.background(this.highlight_color_normal);
			this.highlight_high = false;
		}

		return changed;
	}

	this.number_set = function dt_traceval_number_set(isnumber, decimals)
	{
		this.fixed_number = isnumber;
		if (isnumber) {
			this.fixed_number_decimals = decimals;
			this.fixed_number_decimals_power = Math.pow(10,	decimals);
		}
	}

	this.highlight_set = function dt_traceval_highlight_set(
	    enabled, color_high, color_normal)
	{
		this.highlight_enabled = enabled;
		if (enabled) {
			this.highlight_color_high = color_high;
			this.highlight_color_normal = color_normal;
		}
	}

	this.background = function dt_traceval_background(color)
	{
		this.traceval.style.background = color;
	}


	this.label_set = function dt_traceval_label_set(labeltext)
	{
		this.label = labeltext + ': ';
	}

	this.suffix_set = function dt_traceval_suffix_set(suffixtext)
	{
		this.suffix = ' ' + suffixtext;
	}

	this.trace = new trace();
	this.trace.timeout = 3000;
	
	this.trace.timeout_callback = function trace_timeout_cb(obj)
	{
		this.background("silver");
	}
	
	this.trace.callback = function trace_cb(obj, time, newvalue, unit)
	{
		dt_traceval_this.value(newvalue);
	}

	this.trace_open = function dt_traceval_trace_open(
	    url, name, freq)
	{
		this.value(123);
		this.trace.open(url, name, freq);
	}
}

} /* dt_ui_loaded */
