/* 
	Javascript sending azimuth & elevation speeds 

	Copyright Jeroen Vreeken (pe1rxq@amsat.org), 2008, 2014
	Copyright Stichting C.A. Muller Radioastronomiestation, 2008

	This program is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program.  If not, see <http://www.gnu.org/licenses/>.

 */


var manual_loaded;
if (!manual_loaded) {
manual_loaded = true;

eval(load("dt_ui.js"));


var manual_command_url = "command.cgi_manual";


/*
	Communication with the tracker
 */

function manual_command (url, ident)
{
	this.ident = ident;

	this.open = function (az, el)
	{
		dt_websocket_send("command " + ident + " " +
		    az + " " + el);
	}
}



var manual_slow = 0.0001;
var manual_medium = 0.0003; 
var manual_fast = 0.001;
var manual_speed;


var manual_ind_width = 40;
var manual_ind_height = 40;

var manual_color_down = "blue";
var manual_color_up = "lightblue";

function manual(element_name) {
	var manual_this = this;

	/* 

		Create all the UI elements

	 */

	this.window = new dt_ui_window(element_name);
	
	this.window.title_set("Manual control");
	
	this.ind_up = new dt_ui_value(element_name + "_up");
	this.ind_up.width_set(manual_ind_width);
	this.ind_up.height_set(manual_ind_height);
	this.ind_up.value_set("EL+");
	this.ind_down = new dt_ui_value(element_name + "_down");
	this.ind_down.width_set(manual_ind_width);
	this.ind_down.height_set(manual_ind_height);
	this.ind_down.value_set("EL-");
	this.ind_left = new dt_ui_value(element_name + "_left");
	this.ind_left.width_set(manual_ind_width);
	this.ind_left.height_set(manual_ind_height);
	this.ind_left.value_set("AZ-");
	this.ind_right = new dt_ui_value(element_name + "_right");
	this.ind_right.width_set(manual_ind_width);
	this.ind_right.height_set(manual_ind_height);
	this.ind_right.value_set("AZ+");

	this.ind_up.background_set(manual_color_up);
	this.ind_down.background_set(manual_color_up);
	this.ind_left.background_set(manual_color_up);
	this.ind_right.background_set(manual_color_up);

	this.ind_updown = new dt_ui_element(element_name + "_updown");
	this.ind_updown.add([this.ind_up, this.ind_down]);

	this.ind_leftcont = new dt_ui_element(element_name + "_leftcont");
	this.ind_leftfill = new dt_ui_element(element_name + "_leftfill");
	this.ind_leftfill.width_set(manual_ind_width);
	this.ind_leftfill.height_set(manual_ind_height+2);
	this.ind_leftcont.add([this.ind_leftfill, this.ind_left]);

	this.ind_rightcont = new dt_ui_element(element_name + "_rightcont");
	this.ind_rightfill = new dt_ui_element(element_name + "_rightfill");
	this.ind_rightfill.width_set(manual_ind_width);
	this.ind_rightfill.height_set(manual_ind_height+2);
	this.ind_rightcont.add([this.ind_rightfill, this.ind_right]);

	this.window.add(
	    [this.ind_leftcont, this.ind_updown, this.ind_rightcont]);

	this.ind_leftcont.align_vertical([this.ind_leftfill, this.ind_left]);
	this.ind_rightcont.align_vertical([this.ind_rightfill, this.ind_right]);
	this.ind_updown.align_vertical([this.ind_up, this.ind_down]);

	this.window.align_horizontal(
	    [this.ind_leftcont, this.ind_updown, this.ind_rightcont]);
	
	this.button_speed_fast = new dt_ui_button_indicator(element_name + "_speed_fast");
	this.button_speed_medium = new dt_ui_button_indicator(element_name + "_speed_medium");
	this.button_speed_slow = new dt_ui_button_indicator(element_name + "_speed_slow");

	this.speedbuttons = new	Array(this.button_speed_slow, this.button_speed_medium, this.button_speed_fast);

	this.window.add(this.speedbuttons);

	this.button_speed_fast.text_set("Fast");
	this.button_speed_medium.text_set("Medium");
	this.button_speed_slow.text_set("Slow");
	
	this.window.resize_equal(this.speedbuttons);
	
	this.window.align_horizontal([
	    this.ind_rightcont,
	    this.button_speed_slow
	    ], 10);
	this.window.align_vertical(this.speedbuttons);
	

	/*
	
		UI behaviour
	
	 */


	manual_speed = manual_slow;

	manual_this.button_speed_slow.color_set( 
	    dt_ui_color_indicator_green);
	
	function manual_onclick(elem) {
		manual_this.button_speed_fast.color_set(
		    dt_ui_color_indicator_off);
		manual_this.button_speed_slow.color_set(
		    dt_ui_color_indicator_off);
		manual_this.button_speed_medium.color_set(
		    dt_ui_color_indicator_off);
		
		if (elem == manual_this.button_speed_fast) {
			manual_this.button_speed_fast.color_set(
			    dt_ui_color_indicator_green);
			manual_speed = manual_fast;
		}
		if (elem == manual_this.button_speed_medium) {
			manual_this.button_speed_medium.color_set(
			    dt_ui_color_indicator_green);
			manual_speed = manual_medium;
		}
		if (elem == manual_this.button_speed_slow) {
			manual_this.button_speed_slow.color_set( 
			    dt_ui_color_indicator_green);
			manual_speed = manual_slow;
		}
	}
	this.button_speed_fast.onclick = manual_onclick;
	this.button_speed_medium.onclick = manual_onclick;
	this.button_speed_slow.onclick = manual_onclick;

	var manual_active = true; //todo;

	var manual_elp = 0;
	var manual_elm = 0;
	var manual_azp = 0;
	var manual_azm = 0;

	this.manualcommand = new manual_command(manual_command_url, "manual");

	document.body.onkeydown = function(e)
	{
		if (!manual_active)
			return;
		else {
			var sendcommand = 0;
			if (e.keyCode == 38) {
				sendcommand = 1;
				manual_elp = manual_speed;
				manual_this.ind_up.background_set(
				    manual_color_down);
			}
			if (e.keyCode == 37) {
				sendcommand = 1;
				manual_azm = manual_speed;
				manual_this.ind_left.background_set(
				    manual_color_down);
			}
			if (e.keyCode == 40) {
				sendcommand = 1;
				manual_elm = manual_speed;
				manual_this.ind_down.background_set(
				    manual_color_down);
			}
			if (e.keyCode == 39) {
				sendcommand = 1;
				manual_azp = manual_speed;
				manual_this.ind_right.background_set(
				    manual_color_down);
			}
			if (sendcommand) {
				manual_this.manualcommand.open(
				    manual_azp - manual_azm,
				    manual_elp - manual_elm);
			}
		}
	}
	document.body.onkeyup = function(e)
	{
		if (!manual_active)
			return;
		else {
			var sendcommand = 0;
			if (e.keyCode == 38) {
				sendcommand = 1;
				manual_elp = 0;
				manual_this.ind_up.background_set(
				    manual_color_up);
			}
			if (e.keyCode == 37) {
				sendcommand = 1;
				manual_azm = 0;
				manual_this.ind_left.background_set(
				    manual_color_up);
			}
			if (e.keyCode == 40) {
				sendcommand = 1;
				manual_elm = 0;
				manual_this.ind_down.background_set(
				    manual_color_up);
			}
			if (e.keyCode == 39) {
				sendcommand = 1;
				manual_azp = 0;
				manual_this.ind_right.background_set(
				    manual_color_up);
			}
			
			if (sendcommand) {
				manual_this.manualcommand.open(
				    manual_azp - manual_azm,
				    manual_elp - manual_elm);
			}
		}
	}

	/*
	
		The complete 'manual control' block can be switched on and off.
	
	 */


	this.visible_set = function(visible)
	{
		manual_this.window.visible_set(visible);
		if (!visible) {
			manual_this.manualcommand.open(0, 0);
		}
		manual_this.manual_active = visible;
	}
}



} /* manual_loaded */
