/*
	Copyright Jeroen Vreeken (pe1rxq@amsat.org), 2007
	Copyright Stichting C.A. Muller Radioastronomiestation, 2007

	This program is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program.  If not, see <http://www.gnu.org/licenses/>.

 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <controller/controller_block.h>


/*
       inputs          outputs
       nr name         nr name

     ---------------------------
  ---| 0  in0                  |
     |                 0  out  |----
  ---| 1  in1                  |
     ---------------------------
 */

struct controller_block_private {
	float *input0;
	float *input1;
	float out;
};

static void add_calculate(struct controller_block *sub)
{
	struct controller_block_private *priv = sub->private;

	priv->out = *priv->input0 + *priv->input1;
}

static struct controller_block_interm_list interms[] = {
	{ "in0", CONTROLLER_BLOCK_TERM_FLOAT, offsetof(struct controller_block_private, input0) },
	{ "in1", CONTROLLER_BLOCK_TERM_FLOAT, offsetof(struct controller_block_private, input1) },
	{ NULL }
};

static struct controller_block_outterm_list outterms[] = {
	{ "out", CONTROLLER_BLOCK_TERM_FLOAT, offsetof(struct controller_block_private, out) },
	{ NULL }
};


struct controller_block * block_add_create(char *name)
{
	struct controller_block *add;
	
	if (!(add = controller_block_alloc("add", name, sizeof(struct controller_block_private))))
		return NULL;

	add->private->out = 0.0;
	
	if (controller_block_interm_list_init(add, interms))
		goto err_block;

	if (controller_block_outterm_list_init(add, outterms))
		goto err_block;

	add->calculate = add_calculate;
	add->params = 0;
	add->param = NULL;
	add->param_get = NULL;
	add->param_set = NULL;

	controller_block_add(add);
	return add;

err_block:
	controller_block_free(add);
	return NULL;
}
