/*
	Copyright Jeroen Vreeken (pe1rxq@amsat.org), 2011

	This program is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program.  If not, see <http://www.gnu.org/licenses/>.

 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <controller/controller_block.h>

/*
       inputs      outputs
       nr name     nr name

     -----------------------
     |                     |
  ---| 0  in       0  out0 |----
     |	                   |
     |	           .. ...  |
     |	                   |
     |             n  outn |----
     |	                   |
     -----------------------
     
 */

struct controller_block_private {
	uint32_t *in;
	int      n;
	bool     *out;
};

static void calculate(struct controller_block *dec)
{
	struct controller_block_private *priv = dec->private;
	
	memset(priv->out, 0, sizeof(bool) * priv->n);
	priv->out[*priv->in] = true;
}


struct controller_block * block_decoder_uint32_bool_create(char *name, va_list ap)
{
	struct controller_block *dec;
	int n, i;
	
	n = va_arg(ap, int);

	if (n < 1) {
		printf("Can't create a decoder with %d outputs\n", n);
		return NULL;
	}
		
	dec = malloc(sizeof(struct controller_block));
	if (!dec)
		return NULL;

	dec->type = "decoder_uint32_bool";
	dec->name = malloc(strlen(name)+1);
	if (!dec->name)
		goto err_name;
	strcpy(dec->name, name);

	dec->private = malloc(sizeof(struct controller_block_private));
	if (!dec->private)
		goto err_private;
	dec->private->n = n;

	dec->private->out = calloc(sizeof(bool), n);
	if (!dec->private->out)
		goto err_out;
	
	dec->inputs = 1;
	dec->input = malloc(sizeof(struct controller_block_interm)*n);
	if (!dec->input)
		goto err_input;
	dec->input[0].name = "in";
	dec->input[0].type = CONTROLLER_BLOCK_TERM_UINT32;
	dec->input[0].value.u32 = &dec->private->in;
	dec->input[0].ghostof = NULL;
	
	dec->outputs = n;
	dec->output = malloc(sizeof(struct controller_block_outterm) * n);
	if (!dec->output)
		goto err_output;
	for (i = 0; i < n; i++) {
		dec->output[i].name = malloc(16);
		sprintf(dec->output[i].name, "out%d", i);
		dec->output[i].type = CONTROLLER_BLOCK_TERM_BOOL;
		dec->output[i].value.b = &dec->private->out[i];
		dec->output[i].source = dec;
	}

	dec->calculate = calculate;

	dec->params = 0;
	dec->param = NULL;
	dec->param_get = NULL;
	dec->param_set = NULL;

	controller_block_add(dec);
	return dec;

err_output:
	free(dec->input);
err_input:
	free(dec->private->out);
err_out:
	free(dec->private);
err_private:
	free(dec->name);
err_name:
	free(dec);
	return NULL;
}
