/*
	Copyright Jeroen Vreeken (pe1rxq@amsat.org), 2011

	This program is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program.  If not, see <http://www.gnu.org/licenses/>.

 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <controller/controller_block.h>

/*
       inputs      outputs
       nr name     nr name

     ----------------------
     |                    |
  ---| 0  select   0  out |----
     |	                  |
  ---| 1  in0             |
     |	                  |
     | .. ...	          |
     |	                  |
  ---| n  in(n-1)         |
     |	                  |
     ----------------------
     
 */

struct controller_block_private {
	float    out;
	int      n;
	uint32_t *select;
	float    **in;
};

static void calculate(struct controller_block *mul)
{
	struct controller_block_private *priv = mul->private;
	
	priv->out = *priv->in[*priv->select];
}


struct controller_block * block_multiplexer_create(char *name, va_list ap)
{
	struct controller_block *mul;
	int n, i;
	
	n = va_arg(ap, int);

	mul = malloc(sizeof(struct controller_block));
	if (!mul)
		return NULL;

	mul->type = "multiplexer";
	mul->name = malloc(strlen(name)+1);
	if (!mul->name)
		goto err_name;
	strcpy(mul->name, name);

	mul->private = malloc(sizeof(struct controller_block_private));
	if (!mul->private)
		goto err_private;
	mul->private->out = 0.0;
	
	mul->private->in = calloc(sizeof(float*), n);
	if (!mul->private->in)
		goto err_in;

	mul->inputs = 1 + n;
	mul->input = malloc(sizeof(struct controller_block_interm)*(n+1));
	if (!mul->input)
		goto err_input;
	mul->input[0].name = "select";
	mul->input[0].type = CONTROLLER_BLOCK_TERM_UINT32;
	mul->input[0].value.u32 = &mul->private->select;
	mul->input[0].ghostof = NULL;
	for (i = 0; i < n; i++) {
		mul->input[1+i].name = malloc(16);
		sprintf(mul->input[1+i].name, "in%d", i);
		mul->input[1+i].type = CONTROLLER_BLOCK_TERM_FLOAT;
		mul->input[1+i].value.f = &mul->private->in[i];
		mul->input[1+i].ghostof = NULL;
	}
	
	mul->outputs = 1;
	mul->output = malloc(sizeof(struct controller_block_outterm) * 1);
	if (!mul->output)
		goto err_output;
	mul->output[0].name = "out";
	mul->output[0].type = CONTROLLER_BLOCK_TERM_FLOAT;
	mul->output[0].value.f = &mul->private->out;
	mul->output[0].source = mul;

	mul->calculate = calculate;

	mul->params = 0;
	mul->param = NULL;
	mul->param_get = NULL;
	mul->param_set = NULL;

	controller_block_add(mul);
	return mul;

err_output:
	free(mul->input);
err_input:
	free(mul->private->in);
err_in:
	free(mul->private);
err_private:
	free(mul->name);
err_name:
	free(mul);
	return NULL;
}
