/*
	Copyright Jeroen Vreeken (pe1rxq@amsat.org), 2011
	Copyright Stichting C.A. Muller Radioastronomiestation, 2011

	This program is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program.  If not, see <http://www.gnu.org/licenses/>.

 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <controller/controller_block.h>

/*
       inputs          outputs
       nr name         nr name

     --------------------------
  ---| 0  in                  |
     |                 0  out |----
  ---| 1  on                  |
     --------------------------
 */

struct controller_block_private {
	float *in;
	bool  *on;
	float out;
};

static void switch_calculate(struct controller_block *switchb)
{
	struct controller_block_private *priv = switchb->private;
	float in = *priv->in;
	float out;
	bool on = *priv->on;
	
	out = on ? in : 0.0;

	priv->out = out;
}

static struct controller_block_interm_list interms[] = {
	{ "in", CONTROLLER_BLOCK_TERM_FLOAT, offsetof(struct controller_block_private, in) },
	{ "on", CONTROLLER_BLOCK_TERM_BOOL, offsetof(struct controller_block_private, on) },
	{ NULL }
};

static struct controller_block_outterm_list outterms[] = {
	{ "out", CONTROLLER_BLOCK_TERM_FLOAT, offsetof(struct controller_block_private, out) },
	{ NULL }
};

struct controller_block * block_switch_create(char *name)
{
	struct controller_block *switchb;
	
	if (!(switchb = controller_block_alloc("switch", name, sizeof(struct controller_block_private))))
		return NULL;

	switchb->private->out = 0.0;
	
	if (controller_block_interm_list_init(switchb, interms))
		goto err_block;

	if (controller_block_outterm_list_init(switchb, outterms))
		goto err_block;

	
	switchb->calculate = switch_calculate;

	switchb->params = 0;

	controller_block_add(switchb);
	return switchb;

err_block:
	controller_block_free(switchb);
	return NULL;
}
