/*
	Copyright Jeroen Vreeken (jeroen@vreeken.net), 2014

	This program is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program.  If not, see <http://www.gnu.org/licenses/>.

 */

#ifndef _INCLUDE_CONTROLLER_COMMAND_H_
#define _INCLUDE_CONTROLLER_COMMAND_H_

#include <controller/controller_block.h>

#include <command/command.h>

#define CONTROLLER_COMMAND_QUEUE_DEPTH 128

struct controller_command {
	struct controller_block *block; /* owner */
	char *name; /* outside name */
	char *unit;

	enum command_value_type value_type; /* which type does it accept */
	enum command_ptype command_types[COMMAND_PTYPE_MAX];

	int (*filter)(struct controller_command *command, struct command_entry *entry);
	

	struct command_entry queue[CONTROLLER_COMMAND_QUEUE_DEPTH];
	int queue_w;
	int queue_r;
};

struct controller_command *controller_command_create(
    struct controller_block *block, char *name, char *unit);

void controller_command_destroy(struct controller_command *command);


void controller_command_server_start(int portnr, int max);


static inline int controller_command_queue_read(struct controller_command *command, struct command_entry *entry)
{
	int qr = command->queue_r;
	int qw = command->queue_w;
	
	if (qr != qw) {
		memcpy(entry, &command->queue[qr], sizeof(struct command_entry));
		qr++;
		qr %= CONTROLLER_COMMAND_QUEUE_DEPTH;

		command->queue_r = qr;
		return 0;
	}
	
	return -1;
}

#endif /* _INCLUDE_CONTROLLER_COMMAND_H_ */
