/*
	Copyright Jeroen Vreeken (pe1rxq@amsat.org), 2009, 2013
	Copyright Stichting C.A. Muller Radioastronomiestation, 2009, 2013

	This program is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <stdio.h>
#include <string.h>
#include <stdlib.h>

#include <controller/controller_block.h>
#include <controller/controller_trace.h>

void controller_dumpdot(char *filename)
{
	FILE *fddot;
	int i, j, contextnr;
	char **contextlist = NULL;
	int contextlist_size = 0;
	
	fddot = fopen(filename, "w");
	if (!fddot)
		return;
	
	fprintf(fddot, "digraph controller {\n");
	fprintf(fddot, "\trankdir=LR;\n");
	fprintf(fddot, "\tnode [shape=plaintext];\n");
//	fprintf(fddot, "\tordering=out;\n");
	fprintf(fddot, "\tranksep=1.5;\n");
	fprintf(fddot, "\n");
	
	/* Find all contexts */
	for (i = 0; i < controller_block_nr(); i++) {
		struct controller_block *block = controller_block_get(i);
		bool newcontext = true;
		
		for (j = 0; j < contextlist_size; j++) {
			if (!strcmp(contextlist[j], block->context)) {
				newcontext = false;
				break;
			}
		}
		
		if (newcontext) {
			contextlist_size++;
			
			contextlist = realloc(contextlist, sizeof(char *) * contextlist_size);
			contextlist[contextlist_size-1] = block->context;
		}
	}
	
	for (contextnr = 0; contextnr < contextlist_size; contextnr++) {
	
		fprintf(fddot, "\tsubgraph cluster_%d {\n", contextnr);
		fprintf(fddot, "\t\tlabel=\"%s\";\n", contextlist[contextnr]);
		fprintf(fddot, "\n");
		
		for (i = 0; i < controller_block_nr(); i++) {
			struct controller_block *block;
			int rows = 1;

			block = controller_block_get(i);

			if (strcmp(block->context, contextlist[contextnr]))
				continue;
				
			if (block->inputs > rows)
				rows = block->inputs;
			if (block->outputs > rows)
				rows = block->outputs;

			fprintf(fddot, "\t\t%s [label=<\n", block->name);
			fprintf(fddot, 
			    "\t\t\t<TABLE BORDER=\"0\" CELLBORDER=\"1\" CELLSPACING=\"0\">\n");
			
			for (j = 0; j < rows; j++) {
				fprintf(fddot, "\t\t\t<TR>\n");
				
				if (block->inputs > j) {
					fprintf(fddot, 
					    "\t\t\t\t<TD PORT=\"in_%s\" BGCOLOR=\"lightblue\">%s</TD>\n",
					    block->input[j].name, block->input[j].name);
				} else if (block->outputs > j) {
					fprintf(fddot, "\t\t\t\t<TD BORDER=\"0\"></TD>\n");
				}
				
				if (j == 0) {
					fprintf(fddot, 
					    "\t\t\t\t<TD ROWSPAN=\"%d\" BGCOLOR=\"yellow\">%s<BR/>%s</TD>\n",
					    rows, block->name, block->type);
				}
				
				if (block->outputs > j) {
					fprintf(fddot, 
					    "\t\t\t\t<TD PORT=\"out_%s\" BGCOLOR=\"lightblue\">%s</TD>\n",
					    block->output[j].name, block->output[j].name);
				}

				fprintf(fddot, "\t\t\t\t</TR>\n");
			}
			
			fprintf(fddot, "\t\t\t</TABLE>>];\n");
			fprintf(fddot, "\n");
			
			for (j = 0; j < block->inputs; j++) {
				if (block->input[j].ghostof) {
					fprintf(fddot, 
					    "\t\t%s:in_%s:e -> %s:in_%s:w [color=\"grey\"]\n",
					    block->name, block->input[j].name,
					    block->input[j].ghostof->block->name,
					    block->input[j].ghostof->name);
				}
			}

			for (j = 0; j < block->outputs; j++) {
				if (block->output[j].source != block) {
					fprintf(fddot, 
					    "\t\t%s:out_%s:e -> %s:out_%s:w [color=\"grey\"]\n",
					    block->output[j].source->name,
					    block->output[j].sourceterm->name,
					    block->name, block->output[j].name);
				}
			}
		}

		fprintf(fddot, "\n");

		
		fprintf(fddot, "\t};\n");
		fprintf(fddot, "\n");
	}
		
	for (i = 0; i < controller_block_link_nr(); i++) {
		struct controller_block_link *link = controller_block_link_get(i);
	
		fprintf(fddot, 
		    "\t%s:out_%s:e -> "
		    "%s:in_%s:w "
		    "[%s, labeldistance=2.0, headlabel=<<i>(%s)</i>>, compound=true]\n",
		    link->outblock, link->outterm,
		    link->inblock, link->interm,
		    link->chain ? "color=\"black\"" : "color=\"blue\",constraint=false",
		    link->type ? link->type : "unknown" );
	}
	

	for (i = 0; i < controller_trace_nr(); i++) {
		struct controller_trace_name *trace = controller_trace_get(i);

		fprintf(fddot,
		    "\ttrace_%s [shape=ellipse,label=\"%s\\n[%s]\",color=\"green\"];\n",
		    trace->name, trace->name, trace->unit);
		fprintf(fddot, "\t%s:out_%s:e -> trace_%s [color=\"green\"]\n",
		    trace->blockname, trace->termname, trace->name);
	}
	
	fprintf(fddot, "}\n");
	
	fclose(fddot);
	
	free(contextlist);
	
	return;
}
