/*
	Copyright Jeroen Vreeken (pe1rxq@amsat.org), 2009, 2013
	Copyright Stichting C.A. Muller Radioastronomiestation, 2009, 2013

	This program is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program.  If not, see <http://www.gnu.org/licenses/>.

 */

%option noinput
%option reentrant
%option bison-bridge
%option nounput

%{
#include <stdio.h>
#include <inttypes.h>
#include <controller/controller_load_parser.tab.h>
#include <controller/controller_load_int.h>
#include <log/log.h>

#undef YY_INPUT
#define YY_INPUT(buf,readbytes,sizebytes) controller_load_yy_input(buf,&readbytes,sizebytes, yyscanner)

#define YY_FATAL_ERROR(msg) controller_load_fatal_error( msg , yyscanner)

%}

string		"\""[^"]*"\""

variablename	[a-zA-Z\_][0-9a-zA-Z\-\_]*

doublevariable	"\$("{variablename}")"
stringvariable	"\${"{variablename}"}"

comment		"/*".*"*/"

number		[0-9]+
hexdigit	[0-9a-fA-F]

doublesimple	{number}"."{number} 
doubleexp	{number}"."?{number}?"e"[+-]*{number} 
double		{doublesimple}|{doubleexp}

int		{number}+

unsignedlong	("0x"|"0X"){hexdigit}{5,}

floatcast	"("[ \t]*"float"[ \t]*")"
doublecast	"("[ \t]*"double"[ \t]*")"
intcast		"("[ \t]*"int"[ \t]*")"
boolcast	"("[ \t]*"bool"[ \t]*")"

%%

"{"		{ return BRACEOPENSYM; }
"}"		{ return BRACECLOSESYM; }
"("		{ return PARENTHESESOPENSYM; }
")"		{ return PARENTHESESCLOSESYM; }
","		{ return COMMASYM; }
"*"		{ return STARSYM; }
"/"		{ return SLASHSYM; }
"+"		{ return PLUSSYM; }
"-"		{ return MINSYM; }

"frequency"	{ return FREQUENCYSYM; }
"blocks"	{ return BLOCKSSYM; }
"links"		{ return LINKSSYM; }
"traces"	{ return TRACESSYM; }
"params"	{ return PARAMSSYM; }
"include"	{ return INCLUDESYM; }
"set"		{ return SETSYM; }

"deg2rad"	{ return FUNC_DEG2RAD_SYM; }
"rpm2rads"	{ return FUNC_RPM2RADS_SYM; }
"pi"		{ return CONST_PI_SYM; }

{floatcast} 	{ return FLOATCASTSYM; }
{doublecast} 	{ return DOUBLECASTSYM; }
{intcast}	{ return INTCASTSYM; }
{boolcast}	{ return BOOLCASTSYM; }

"true"		{ yylval->integer = 1; return BOOLSYM; }
"false"		{ yylval->integer = 0; return BOOLSYM; }

{string}	{ 
			yylval->string = strdup(yytext + 1);
			yylval->string[strlen(yylval->string)-1] = 0;
			return STRINGSYM;
		}

{doublevariable}	{
			yylval->string = strdup(yytext + 2);
			yylval->string[strlen(yylval->string)-1] = 0;
			return DOUBLEVARIABLESYM;
		}

{stringvariable}	{
			yylval->string = strdup(yytext + 2);
			yylval->string[strlen(yylval->string)-1] = 0;
			return STRINGVARIABLESYM;
		}


{double}	{
			sscanf(yytext, "%lg", &yylval->dbl);
			return DOUBLESYM;
		}

{unsignedlong}	{
			sscanf(yytext, "0x%lx", &yylval->ul);
			return UNSIGNEDLONGSYM;
		}

{int}		{
			yylval->integer = atoi(yytext);
			return INTSYM;
		}

{variablename}	{
			yylval->string = strdup(yytext);
			return VARIABLENAMESYM;
		}



{comment}	/* skip */
[ \t\r]		/* skip */
#.*		/* skip */

"\n"		{ yyset_lineno(yyget_lineno(yyscanner)+1, yyscanner);}

%%

int yywrap(yyscan_t scanner) { return 1; }
