/*
	Copyright Jeroen Vreeken (pe1rxq@amsat.org), 2009
	Copyright Stichting C.A. Muller Radioastronomiestation, 2009

	This program is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program.  If not, see <http://www.gnu.org/licenses/>.

 */

#include <stdlib.h>
#include <string.h>

#include <controller/controller_load.h>

enum variable_type {
	VARIABLE_STRING,
	VARIABLE_DOUBLE,
};

struct variable {
	enum variable_type type;
	char *name;
	
	double doubleval;
	char *stringval;
	
	struct variable *next;
};

static struct variable *variables = NULL;



double controller_load_variable_double_get(char *varname)
{
	struct variable *entry;
	
	for (entry = variables; entry; entry = entry->next) {
		if (!strcmp(entry->name, varname) &&
		    entry->type == VARIABLE_DOUBLE)
			return entry->doubleval;
	}
	
	return 0.0;
}

void controller_load_variable_double_set(char *varname, double val)
{
	struct variable *entry;
	
	for (entry = variables; entry; entry = entry->next) {
		if (!strcmp(entry->name, varname)) {
			if (entry->type == VARIABLE_STRING) {
				free(entry->stringval);
			}
			break;
		}
	}

	if (!entry) {
		entry = calloc(sizeof(struct variable), 1);
		entry->name = strdup(varname);
		
		entry->next = variables;
		variables = entry;
	}

	entry->type = VARIABLE_DOUBLE;
	entry->doubleval = val;
}

char * controller_load_variable_string_get(char *varname)
{
	struct variable *entry;
	
	for (entry = variables; entry; entry = entry->next) {
		if (!strcmp(entry->name, varname) &&
		    entry->type == VARIABLE_STRING)
			return entry->stringval;
	}
	
	return "";
}

void controller_load_variable_string_set(char *varname, char *val)
{
	struct variable *entry;
	
	for (entry = variables; entry; entry = entry->next) {
		if (!strcmp(entry->name, varname)) {
			if (entry->type == VARIABLE_STRING) {
				free(entry->stringval);
			}
			break;
		}
	}

	if (!entry) {
		entry = calloc(sizeof(struct variable), 1);
		entry->name = strdup(varname);
		
		entry->next = variables;
		variables = entry;
	}

	entry->type = VARIABLE_STRING;
	entry->stringval = strdup(val);
}

