/*
	Copyright Jeroen Vreeken (pe1rxq@amsat.org), 2009, 2013
	Copyright Stichting C.A. Muller Radioastronomiestation, 2009

	This program is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program.  If not, see <http://www.gnu.org/licenses/>.

 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <controller/controller_block.h>

#include <ec/block_beckhoff_el4xxx.h>
#include <ec/ec.h>
#include <ec/esc.h>
#include <ec/esc_esi.h>
#include <ec/esc_coe.h>
#include <ec/esc_id.h>
#include <log/log.h>

struct controller_block_private {
	float	*output[8];

	float gain;
	float offset;
	
	unsigned char *tx_buffer;

	struct esc_device *dev;
};

static void calculate_1u(struct controller_block *block)
{
	struct controller_block_private *priv = block->private;
	float value1;
	struct {
		uint16_t value1;
	} __packed *out;
	
	value1 = (priv->offset + *priv->output[0]) * priv->gain;
	if (value1 > 65535)
		value1 = 65535;
	
	out = (void*)priv->tx_buffer;
	
	out->value1 = htole16(value1);

	ec_tx_pdo(controller_samplenr);
}

static void calculate_1s(struct controller_block *block)
{
	struct controller_block_private *priv = block->private;
	float value1;
	struct {
		int16_t value1;
	} __packed *out;
	
	value1 = (priv->offset + *priv->output[0]) * priv->gain;
	if (value1 > 32767)
		value1 = 32767;
	if (value1 < -32768)
		value1 = -32768;
	
	out = (void*)priv->tx_buffer;
	
	out->value1 = htole16((int16_t)value1);

	ec_tx_pdo(controller_samplenr);
}

static void calculate_2u(struct controller_block *block)
{
	struct controller_block_private *priv = block->private;
	float value1, value2;
	struct {
		uint16_t value1;
		uint16_t value2;
	} __packed *out;
	
	value1 = (priv->offset + *priv->output[0]) * priv->gain;
	if (value1 > 65535)
		value1 = 65535;
	value2 = (priv->offset + *priv->output[1]) * priv->gain;
	if (value2 > 65535)
		value2 = 65535;
	
	out = (void*)priv->tx_buffer;
	
	out->value1 = htole16(value1);
	out->value2 = htole16(value2);

	ec_tx_pdo(controller_samplenr);
}

static void calculate_2s(struct controller_block *block)
{
	struct controller_block_private *priv = block->private;
	float value1, value2;
	struct {
		int16_t value1;
		int16_t value2;
	} __packed *out;
	
	value1 = (priv->offset + *priv->output[0]) * priv->gain;
	value2 = (priv->offset + *priv->output[1]) * priv->gain;
	if (value1 > 32767)
		value1 = 32767;
	if (value1 < -32768)
		value1 = -32768;
	if (value2 > 32767)
		value2 = 32767;
	if (value2 < -32768)
		value2 = -32768;
	
	out = (void*)priv->tx_buffer;
	
	out->value1 = htole16((int16_t)value1);
	out->value2 = htole16((int16_t)value2);

	ec_tx_pdo(controller_samplenr);
}

static void calculate_4u(struct controller_block *block)
{
	struct controller_block_private *priv = block->private;
	float value1, value2, value3, value4;
	struct {
		uint16_t value1;
		uint16_t value2;
		uint16_t value3;
		uint16_t value4;
	} __packed *out;
	
	value1 = (priv->offset + *priv->output[0]) * priv->gain;
	if (value1 > 65535)
		value1 = 65535;
	value2 = (priv->offset + *priv->output[1]) * priv->gain;
	if (value2 > 65535)
		value2 = 65535;
	value3 = (priv->offset + *priv->output[2]) * priv->gain;
	if (value3 > 65535)
		value3 = 65535;
	value4 = (priv->offset + *priv->output[3]) * priv->gain;
	if (value4 > 65535)
		value4 = 65535;
	
	out = (void*)priv->tx_buffer;
	
	out->value1 = htole16(value1);
	out->value2 = htole16(value2);
	out->value3 = htole16(value3);
	out->value4 = htole16(value4);

	ec_tx_pdo(controller_samplenr);
}

static void calculate_4s(struct controller_block *block)
{
	struct controller_block_private *priv = block->private;
	float value1, value2, value3, value4;
	struct {
		int16_t value1;
		int16_t value2;
		int16_t value3;
		int16_t value4;
	} __packed *out;
	
	value1 = (priv->offset + *priv->output[0]) * priv->gain;
	value2 = (priv->offset + *priv->output[1]) * priv->gain;
	value3 = (priv->offset + *priv->output[2]) * priv->gain;
	value4 = (priv->offset + *priv->output[3]) * priv->gain;
	if (value1 > 32767)
		value1 = 32767;
	if (value1 < -32768)
		value1 = -32768;
	if (value2 > 32767)
		value2 = 32767;
	if (value2 < -32768)
		value2 = -32768;
	if (value3 > 32767)
		value3 = 32767;
	if (value3 < -32768)
		value3 = -32768;
	if (value4 > 32767)
		value4 = 32767;
	if (value4 < -32768)
		value4 = -32768;
	
	out = (void*)priv->tx_buffer;
	
	out->value1 = htole16((int16_t)value1);
	out->value2 = htole16((int16_t)value2);
	out->value3 = htole16((int16_t)value3);
	out->value4 = htole16((int16_t)value4);

	ec_tx_pdo(controller_samplenr);
}

static void calculate_8u(struct controller_block *block)
{
	struct controller_block_private *priv = block->private;
	float value1, value2, value3, value4;
	float value5, value6, value7, value8;
	struct {
		uint16_t value1;
		uint16_t value2;
		uint16_t value3;
		uint16_t value4;
		uint16_t value5;
		uint16_t value6;
		uint16_t value7;
		uint16_t value8;
	} __packed *out;
	
	value1 = (priv->offset + *priv->output[0]) * priv->gain;
	if (value1 > 65535)
		value1 = 65535;
	value2 = (priv->offset + *priv->output[1]) * priv->gain;
	if (value2 > 65535)
		value2 = 65535;
	value3 = (priv->offset + *priv->output[2]) * priv->gain;
	if (value3 > 65535)
		value3 = 65535;
	value4 = (priv->offset + *priv->output[3]) * priv->gain;
	if (value4 > 65535)
		value4 = 65535;
	value5 = (priv->offset + *priv->output[4]) * priv->gain;
	if (value5 > 65535)
		value5 = 65535;
	value6 = (priv->offset + *priv->output[5]) * priv->gain;
	if (value6 > 65535)
		value6 = 65535;
	value7 = (priv->offset + *priv->output[6]) * priv->gain;
	if (value7 > 65535)
		value7 = 65535;
	value8 = (priv->offset + *priv->output[7]) * priv->gain;
	if (value8 > 65535)
		value8 = 65535;
	
	out = (void*)priv->tx_buffer;
	
	out->value1 = htole16(value1);
	out->value2 = htole16(value2);
	out->value3 = htole16(value3);
	out->value4 = htole16(value4);
	out->value5 = htole16(value5);
	out->value6 = htole16(value6);
	out->value7 = htole16(value7);
	out->value8 = htole16(value8);

	ec_tx_pdo(controller_samplenr);
}

static void calculate_8s(struct controller_block *block)
{
	struct controller_block_private *priv = block->private;
	float value1, value2, value3, value4;
	float value5, value6, value7, value8;
	struct {
		int16_t value1;
		int16_t value2;
		int16_t value3;
		int16_t value4;
		int16_t value5;
		int16_t value6;
		int16_t value7;
		int16_t value8;
	} __packed *out;
	
	value1 = (priv->offset + *priv->output[0]) * priv->gain;
	value2 = (priv->offset + *priv->output[1]) * priv->gain;
	value3 = (priv->offset + *priv->output[2]) * priv->gain;
	value4 = (priv->offset + *priv->output[3]) * priv->gain;
	value5 = (priv->offset + *priv->output[4]) * priv->gain;
	value6 = (priv->offset + *priv->output[5]) * priv->gain;
	value7 = (priv->offset + *priv->output[6]) * priv->gain;
	value8 = (priv->offset + *priv->output[7]) * priv->gain;
	if (value1 > 32767)
		value1 = 32767;
	if (value1 < -32768)
		value1 = -32768;
	if (value2 > 32767)
		value2 = 32767;
	if (value2 < -32768)
		value2 = -32768;
	if (value3 > 32767)
		value3 = 32767;
	if (value3 < -32768)
		value3 = -32768;
	if (value4 > 32767)
		value4 = 32767;
	if (value4 < -32768)
		value4 = -32768;
	if (value5 > 32767)
		value5 = 32767;
	if (value5 < -32768)
		value5 = -32768;
	if (value6 > 32767)
		value6 = 32767;
	if (value6 < -32768)
		value6 = -32768;
	if (value7 > 32767)
		value7 = 32767;
	if (value7 < -32768)
		value7 = -32768;
	if (value8 > 32767)
		value8 = 32767;
	if (value8 < -32768)
		value8 = -32768;
	
	out = (void*)priv->tx_buffer;
	
	out->value1 = htole16((int16_t)value1);
	out->value2 = htole16((int16_t)value2);
	out->value3 = htole16((int16_t)value3);
	out->value4 = htole16((int16_t)value4);
	out->value5 = htole16((int16_t)value5);
	out->value6 = htole16((int16_t)value6);
	out->value7 = htole16((int16_t)value7);
	out->value8 = htole16((int16_t)value8);

	ec_tx_pdo(controller_samplenr);
}

static struct controller_block_interm_list interms_1[] = {
	{ "output1", CONTROLLER_BLOCK_TERM_FLOAT, offsetof(struct controller_block_private, output[0])  },
	{ NULL },
};

static struct controller_block_interm_list interms_2[] = {
	{ "output1", CONTROLLER_BLOCK_TERM_FLOAT, offsetof(struct controller_block_private, output[0])  },
	{ "output2", CONTROLLER_BLOCK_TERM_FLOAT, offsetof(struct controller_block_private, output[1])  },
	{ NULL },
};

static struct controller_block_interm_list interms_4[] = {
	{ "output1", CONTROLLER_BLOCK_TERM_FLOAT, offsetof(struct controller_block_private, output[0])  },
	{ "output2", CONTROLLER_BLOCK_TERM_FLOAT, offsetof(struct controller_block_private, output[1])  },
	{ "output3", CONTROLLER_BLOCK_TERM_FLOAT, offsetof(struct controller_block_private, output[2])  },
	{ "output4", CONTROLLER_BLOCK_TERM_FLOAT, offsetof(struct controller_block_private, output[3])  },
	{ NULL },
};

static struct controller_block_interm_list interms_8[] = {
	{ "output1", CONTROLLER_BLOCK_TERM_FLOAT, offsetof(struct controller_block_private, output[0])  },
	{ "output2", CONTROLLER_BLOCK_TERM_FLOAT, offsetof(struct controller_block_private, output[1])  },
	{ "output3", CONTROLLER_BLOCK_TERM_FLOAT, offsetof(struct controller_block_private, output[2])  },
	{ "output4", CONTROLLER_BLOCK_TERM_FLOAT, offsetof(struct controller_block_private, output[3])  },
	{ "output5", CONTROLLER_BLOCK_TERM_FLOAT, offsetof(struct controller_block_private, output[4])  },
	{ "output6", CONTROLLER_BLOCK_TERM_FLOAT, offsetof(struct controller_block_private, output[5])  },
	{ "output7", CONTROLLER_BLOCK_TERM_FLOAT, offsetof(struct controller_block_private, output[6])  },
	{ "output8", CONTROLLER_BLOCK_TERM_FLOAT, offsetof(struct controller_block_private, output[7])  },
	{ NULL },
};

struct beckhoff_el4xxx_type {
	char *name;
	uint32_t productcode;
	int outputs;
	float gain;
	float offset;
	bool sign;
	struct controller_block_interm_list *interms;
};

static struct beckhoff_el4xxx_type devlist[] = {
	{ "EL4001", ESC_ESI_PRODUCTCODE_BECKHOFF_EL4001, 1, 4096/10.0,  0.0,   false, interms_1 },
	{ "EL4002", ESC_ESI_PRODUCTCODE_BECKHOFF_EL4002, 2, 4096/10.0,  0.0,   false, interms_2 },
	{ "EL4004", ESC_ESI_PRODUCTCODE_BECKHOFF_EL4004, 4, 4096/10.0,  0.0,   false, interms_4 },
	{ "EL4008", ESC_ESI_PRODUCTCODE_BECKHOFF_EL4008, 8, 4096/10.0,  0.0,   false, interms_8 },
	{ "EL4011", ESC_ESI_PRODUCTCODE_BECKHOFF_EL4001, 1, 4096/0.02,  0.0,   false, interms_1 },
	{ "EL4012", ESC_ESI_PRODUCTCODE_BECKHOFF_EL4002, 2, 4096/0.02,  0.0,   false, interms_2 },
	{ "EL4014", ESC_ESI_PRODUCTCODE_BECKHOFF_EL4004, 4, 4096/0.02,  0.0,   false, interms_4 },
	{ "EL4018", ESC_ESI_PRODUCTCODE_BECKHOFF_EL4008, 8, 4096/0.02,  0.0,   false, interms_8 },
	{ "EL4021", ESC_ESI_PRODUCTCODE_BECKHOFF_EL4001, 1, 4096/0.016, -0.004, false, interms_1 },
	{ "EL4022", ESC_ESI_PRODUCTCODE_BECKHOFF_EL4002, 2, 4096/0.016, -0.004, false, interms_2 },
	{ "EL4024", ESC_ESI_PRODUCTCODE_BECKHOFF_EL4004, 4, 4096/0.016, -0.004, false, interms_4 },
	{ "EL4028", ESC_ESI_PRODUCTCODE_BECKHOFF_EL4008, 8, 4096/0.016, -0.004, false, interms_8 },
	{ "EL4001", ESC_ESI_PRODUCTCODE_BECKHOFF_EL4001, 1, 2048/10.0,  0.0,   true, interms_1  },
	{ "EL4002", ESC_ESI_PRODUCTCODE_BECKHOFF_EL4002, 2, 2048/10.0,  0.0,   true, interms_2  },
	{ "EL4004", ESC_ESI_PRODUCTCODE_BECKHOFF_EL4004, 4, 2048/10.0,  0.0,   true, interms_4  },
	{ "EL4008", ESC_ESI_PRODUCTCODE_BECKHOFF_EL4008, 8, 2048/10.0,  0.0,   true, interms_8  },
	{ "EL4102", ESC_ESI_PRODUCTCODE_BECKHOFF_EL4102, 2, 65536/10.0, 0.0,   false, interms_2 },
	{ "EL4112", ESC_ESI_PRODUCTCODE_BECKHOFF_EL4112, 2, 65536/0.02, 0.0,   false, interms_2 },
	{ "EL4114", ESC_ESI_PRODUCTCODE_BECKHOFF_EL4114, 4, 65536/0.02, 0.0,   false, interms_4 },
	{ "EL4122", ESC_ESI_PRODUCTCODE_BECKHOFF_EL4122, 2, 65536/0.016, 0.0,  false, interms_2 },
	{ "EL4132", ESC_ESI_PRODUCTCODE_BECKHOFF_EL4132, 2, 32768/10.0, 0.0,   true, interms_2  },
	{ "EL4134", ESC_ESI_PRODUCTCODE_BECKHOFF_EL4134, 4, 32768/10.0, 0.0,   true, interms_4  },
};

struct controller_block * block_beckhoff_el4xxx_create(char *name, va_list ap)
{
	struct controller_block *block;
	struct ec_dgram_addr addr;
	int i;
	int dev = -1;
	char *eltype;
	int devno;
	struct timespec timeout = { 1, 0 };
		
	uint32_t vendor, product;

	eltype = va_arg(ap, char *);
	devno = va_arg(ap, int);
	
	for (i = 0; i < sizeof(devlist)/sizeof(struct beckhoff_el4xxx_type); i++) {
		if (!strcmp(eltype, devlist[i].name)) {
			dev = i;
			break;
		}
	}
	if (dev < 0)
		return NULL;
	
	ec_addr_set_auto_inc_nr(&addr, devno);
	vendor = esc_esi_vendorid_get(&addr);
	product = esc_esi_productcode_get(&addr);

	if (vendor != ESC_ESI_VENDORID_BECKHOFF ||
	    product != devlist[i].productcode) {
		log_send(LOG_T_ERROR, "%s: Device not found\n", name);
		return NULL;
	}

	
	block = controller_block_alloc("beckhoff_el4xxx", name, 
	    sizeof(struct controller_block_private));
	if (!block)
		return NULL;

	block->private->dev = esc_device_create(&addr, name);
	esc_esi_device_fill(block->private->dev);
	esc_device_initialize_pre_operational(block->private->dev);
	
	block->private->tx_buffer = block->private->dev->tx_data;

	if (block->private->dev->sm[block->private->dev->tx_pdo_sm].len !=
	    0x2 * devlist[dev].outputs) {
		log_send(LOG_T_ERROR, "TX PDO length mismatch");
	}
	
	esc_coe_create(block->private->dev->mailbox);

//	esc_dc_init(&addr);
//	esc_dc_sync0_cycle_time(&addr, 1000*1000);

/*
leval16 = 0x4;
addr.addr.position.off = ESC_ADDR_MAP_FMMU_NR(0) | ESC_ADDR_MAP_FMMU_LENGTH;
ec_datagram_write(&addr, &leval16, 2);

leval16 = 0x1000;
addr.addr.position.off = ESC_ADDR_MAP_FMMU_NR(0) | ESC_ADDR_MAP_FMMU_PHYSICAL_START;
ec_datagram_write(&addr, &leval16, 2);

val8 = ESC_FMMU_TYPE_RW;
addr.addr.position.off = ESC_ADDR_MAP_FMMU_NR(0) | ESC_ADDR_MAP_FMMU_TYPE;
ec_datagram_write(&addr, &val8, 1);

val8 = ESC_FMMU_ACTIVATE_ACTIVATED;
addr.addr.position.off = ESC_ADDR_MAP_FMMU_NR(0) | ESC_ADDR_MAP_FMMU_ACTIVATE;
ec_datagram_write(&addr, &val8, 1);

leval16 = 0x4;
addr.addr.position.off = ESC_ADDR_MAP_FMMU_NR(1) | ESC_ADDR_MAP_FMMU_LENGTH;
ec_datagram_write(&addr, &leval16, 2);

leval16 = 0x1100;
addr.addr.position.off = ESC_ADDR_MAP_FMMU_NR(1) | ESC_ADDR_MAP_FMMU_PHYSICAL_START;
ec_datagram_write(&addr, &leval16, 2);
val8 = ESC_FMMU_TYPE_RW;
addr.addr.position.off = ESC_ADDR_MAP_FMMU_NR(1) | ESC_ADDR_MAP_FMMU_TYPE;
ec_datagram_write(&addr, &val8, 1);
val8 = ESC_FMMU_ACTIVATE_ACTIVATED;
addr.addr.position.off = ESC_ADDR_MAP_FMMU_NR(1) | ESC_ADDR_MAP_FMMU_ACTIVATE;
ec_datagram_write(&addr, &val8, 1);


	addr.addr.position.off = ESC_ADDR_MAP_DC_ACTIVATION_REGISTER;
	val8 = 0x00;
	ec_datagram_write(&addr, &val8, 1);


addr.addr.position.off = ESC_ADDR_MAP_WATCHDOG_TIME_PD;
leval16 = 0x0000;
ec_datagram_write(&addr, &leval16, 2);


addr.addr.position.off = ESC_ADDR_MAP_WATCHDOG_COUNTER_PD;
val8 = 0;
ec_datagram_write(&addr, &val8, 1);
*/

	if (esc_al_state_set(&addr, ESC_AL_STATE_SAFE_OPERATIONAL, &timeout) < 0)
		log_send(LOG_T_ERROR, 
		    "%s: Could not go to state safe operational", name);

	if (esc_al_state_set(&addr, ESC_AL_STATE_OPERATIONAL, &timeout) < 0)
		log_send(LOG_T_ERROR, "%s: Could not go to state operational",
		name);
	log_send(LOG_T_DEBUG, "%s: state: %d", name, esc_al_state_get(&addr));
	log_send(LOG_T_DEBUG, "%s: statuscode: %d",
	    name, esc_al_status_code_get(&addr));

	/* Inputs on the hardware are outputs on this software block */
	if (controller_block_interm_list_init(block, devlist[dev].interms))
		return NULL;

	switch (devlist[dev].outputs) {
		case 1:
			if (devlist[dev].sign)
				block->calculate = calculate_1s;
			else
				block->calculate = calculate_1u;
			break;
		case 2:
			if (devlist[dev].sign)
				block->calculate = calculate_2s;
			else
				block->calculate = calculate_2u;
			break;
		case 4:
			if (devlist[dev].sign)
				block->calculate = calculate_4s;
			else
				block->calculate = calculate_4u;
			break;
		case 8:
			if (devlist[dev].sign)
				block->calculate = calculate_8s;
			else
				block->calculate = calculate_8u;
			break;
	}
	block->private->gain = devlist[dev].gain;
	block->private->offset = devlist[dev].offset;
	
	controller_block_add(block);
	return block;
}
