/*
	Copyright Jeroen Vreeken (pe1rxq@amsat.org), 2009
	Copyright Stichting C.A. Muller Radioastronomiestation, 2009

	This program is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program.  If not, see <http://www.gnu.org/licenses/>.

 */

#include <stdio.h>
#include <stdlib.h>
#include <stdint.h>
#include <string.h>

#include <controller/controller_block.h>

#include <ec/ec.h>
#include <ec/esc.h>
#include <ec/esc_esi.h>
#include <ec/esc_coe.h>
#include <ec/esc_id.h>
#include <log/log.h>

struct controller_block_private {
	float	 converted_value;
	uint32_t value;
	
	double 	gain;
	double 	offset;
	
	unsigned char *rx_buffer;
	
	struct esc_device *dev;
};

static void calculate(struct controller_block *block)
{
	struct controller_block_private *priv = block->private;
	struct {
		uint8_t	 status;
		uint32_t value;
	} __packed *rxdata;
	uint32_t value;
	
	rxdata = (void*)priv->rx_buffer;
	
	ec_rx_pdo(controller_samplenr);

	value = le32toh(rxdata->value);
	priv->value = value;
	priv->converted_value = (value + priv->gain) / priv->offset;
}

static struct controller_block_outterm_list outterms[] = {
	{ "converted_value", CONTROLLER_BLOCK_TERM_FLOAT,  offsetof(struct controller_block_private, converted_value)  },
	{ "value",           CONTROLLER_BLOCK_TERM_UINT32, offsetof(struct controller_block_private, value) },
	{ NULL },
};

struct controller_block * block_beckhoff_el5001_create(char *name, va_list ap)
{
	struct controller_block *block;
	struct ec_dgram_addr addr;
	int devno;
	struct timespec timeout = { 1, 0 };
	uint32_t vendor, product;

	devno = va_arg(ap, int);
	
	ec_addr_set_auto_inc_nr(&addr, devno);
	vendor = esc_esi_vendorid_get(&addr);
	product = esc_esi_productcode_get(&addr);
	if (vendor != ESC_ESI_VENDORID_BECKHOFF ||
	    product != ESC_ESI_PRODUCTCODE_BECKHOFF_EL5001) {
		log_send(LOG_T_ERROR, "%s: Device not found\n", name);
		return NULL;
	}
	
	block = controller_block_alloc("beckhoff_el5001", name,
	    sizeof(struct controller_block_private));
	if (!block)
		return NULL;

	block->private->dev = esc_device_create(&addr, name);
	esc_esi_device_fill(block->private->dev);
	esc_device_initialize_pre_operational(block->private->dev);
	
	block->private->rx_buffer = block->private->dev->rx_data;

	esc_coe_create(block->private->dev->mailbox);

	if (esc_al_state_set(&addr, ESC_AL_STATE_SAFE_OPERATIONAL, &timeout) < 0)
		log_send(LOG_T_ERROR, 
		    "%s: Could not go to state safe operational", name);
	if (esc_al_state_set(&addr, ESC_AL_STATE_OPERATIONAL, &timeout) < 0)
		log_send(LOG_T_ERROR, 
		    "%s: Could not go to state operational", name);

	/* Inputs on the hardware are outputs on this software block */
	if (controller_block_outterm_list_init(block, outterms))
		return NULL;

	block->calculate = calculate;
	
	block->private->gain = 1.0;
	block->private->offset = 0.0;
	
	controller_block_add(block);
	return block;
}
