/*
	Copyright Jeroen Vreeken (pe1rxq@amsat.org), 2009 - 2013
	Copyright Stichting C.A. Muller Radioastronomiestation, 2009 - 2013

	This program is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program.  If not, see <http://www.gnu.org/licenses/>.

 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <controller/controller_block.h>

#include <ec/block_ec.h>
#include <ec/ec.h>
#include <log/log.h>

static struct controller_block *ec_block = NULL;

static void calculate(struct controller_block *ec)
{
	ec_datagram_sample();
}

static void param_set(struct controller_block *spg, int param, va_list val)
{
	int enabled = va_arg(val, int);

	ec_rx_pdo_enable(enabled);
	ec_tx_pdo_enable(enabled);
}

static struct controller_block_param_list params[] = {
	{ "pdo_enabled", true },
	{ NULL },
};

struct controller_block * block_ec_create(char *name, va_list ap)
{
	struct controller_block *ec;
	int ret;
	char *ifname;
	int nr_slaves;
	int single_cycle;
	
	/* For now we only support a single ethercat interface,
	   but we allow multiple inits
	 */
	if (ec_block) {
		return ec_block;
	}

	ifname = va_arg(ap, char *);
	nr_slaves = va_arg(ap, int);
	single_cycle = va_arg(ap, int);
	
	ret = ec_init(ifname, single_cycle);

	if (ret != 0)
		return NULL;

	while ((ret = ec_slave_count()) != nr_slaves) {
		if (ret == -1) {
			log_send(LOG_T_ERROR, "Error counting slaves");
		} else if (ret < nr_slaves) {
			log_send(LOG_T_ERROR, 
			   "Found only %d slaves, waiting for %d.",
			   ret, nr_slaves);
		} else {
			log_send(LOG_T_WARNING,
			   "More slaves found (%d) than expected (%d).",
			   ret, nr_slaves);
			break;
		}
		sleep(10);
	}

	ec = controller_block_alloc("ec", name, 0);
	if (!ec)
		goto err_ec;

	ec->calculate = calculate;

	if (controller_block_param_list_init(ec, params))
		goto err_params;

	ec->param_get = NULL;
	ec->param_set = param_set;

	controller_block_add(ec);

	ec_block = ec;

	return ec;

err_params:
	free(ec);
err_ec:
	return NULL;
}
