/*
	Copyright Jeroen Vreeken (pe1rxq@amsat.org), 2009, 2011, 2013
	Copyright Stichting C.A. Muller Radioastronomiestation, 2009, 2011

	This program is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program.  If not, see <http://www.gnu.org/licenses/>.

 */

#ifndef _INCLUDE_EC_ESC_H_
#define _INCLUDE_EC_ESC_H_


#include <unistd.h>
#include <time.h>
#include <ec/ec.h>

#include <ec/esc_sm.h>

int esc_init(struct ec_dgram_addr *addr);


enum esc_al_state {
	ESC_AL_STATE_ERROR = 			-1, /* error getting state */
	ESC_AL_STATE_INIT = 			1,
	ESC_AL_STATE_REQUEST_BOOTSTRAP = 	3,
	ESC_AL_STATE_PRE_OPERATIONAL = 		2,
	ESC_AL_STATE_SAFE_OPERATIONAL = 	4,
	ESC_AL_STATE_OPERATIONAL = 		8,
};

static inline char *esc_al_state2str(enum esc_al_state state)
{
	char *str;

	switch(state) {
		case ESC_AL_STATE_ERROR:
			str = "Error, state unknown";
			break;
		case ESC_AL_STATE_INIT:
			str = "Init";
			break;
		case ESC_AL_STATE_REQUEST_BOOTSTRAP:
			str = "Request Bootstrap";
			break;
		case ESC_AL_STATE_PRE_OPERATIONAL:
			str = "Pre-Operational";
			break;
		case ESC_AL_STATE_SAFE_OPERATIONAL:
			str = "Safe-Operational";
			break;
		case ESC_AL_STATE_OPERATIONAL:
			str = "Operational";
			break;
		default:
			str = "unknown";
			break;
	}
	return str;
}

enum esc_al_state esc_al_state_get(struct ec_dgram_addr *addr);
int esc_al_state_set(struct ec_dgram_addr *addr, enum esc_al_state newstate,
    struct timespec *timeout);

enum esc_al_status {
	ESC_AL_STATUS_ERROR = 		-1, /* error getting status */
	ESC_AL_STATUS_NO_ERROR =	0x0000,
	ESC_AL_STATUS_UNSPECIFIED =	0x0001,
	ESC_AL_STATUS_INVALID_REQ =	0x0011,
	ESC_AL_STATUS_UNKNOWN_REQ =	0x0012,
	ESC_AL_STATUS_BOOTSTRAP_UNSUP =	0x0013,
	ESC_AL_STATUS_NO_VALID_FMW =	0x0014,
	ESC_AL_STATUS_INVALID_MBOX1 =	0x0015,
	ESC_AL_STATUS_INVALID_MBOX2 =	0x0016,
	ESC_AL_STATUS_INVALID_SM =	0x0017,
	ESC_AL_STATUS_NO_VALID_INPUT =	0x0018,
	ESC_AL_STATUS_NO_VALID_OUTPUT =	0x0019,
	ESC_AL_STATUS_SYNC_ERROR =	0x001a,
	ESC_AL_STATUS_SM_WATCHDOG =	0x001b,
	ESC_AL_STATUS_INVALID_SM_TYPES =0x001c,
	ESC_AL_STATUS_INVALID_OUT_CONF =0x001d,
	ESC_AL_STATUS_INVALID_IN_CONF =	0x001e,
	ESC_AL_STATUS_INVALID_WD_CONF =	0x001f,
	ESC_AL_STATUS_SLAVE_COLD =	0x0020,
	ESC_AL_STATUS_SLAVE_INIT =	0x0021,
	ESC_AL_STATUS_SLAVE_PREOP =	0x0022,
	ESC_AL_STATUS_SLAVE_SAFEOP =	0x0023,
	ESC_AL_STATUS_INVALID_DC_S_CONF=0x0030,
	ESC_AL_STATUS_INVALID_DC_L_CONF=0x0031,
	ESC_AL_STATUS_PLL_ERROR =	0x0032,
	ESC_AL_STATUS_INVALID_DC_IO =	0x0033,
	ESC_AL_STATUS_INVALID_DC_TO =	0x0034,
	ESC_AL_STATUS_DC_SYNC0_CYCLE =	0x0036,
	ESC_AL_STATUS_DC_SYNC1_CYCLE = 	0x0037,
	ESC_AL_STATUS_MBOX_EOE =	0x0042,
	ESC_AL_STATUS_MBOX_COE =	0x0043,
	ESC_AL_STATUS_MBOX_FOE =	0x0044,
	ESC_AL_STATUS_MBOX_SOE =	0x0045,
	ESC_AL_STATUS_MBOX_VOE =	0x004f,
};

static inline char *esc_al_status2str(enum esc_al_status status)
{
	char *str;

	switch(status) {
		case ESC_AL_STATUS_ERROR:
			str = "Error, status unknown";
			break;
		case ESC_AL_STATUS_NO_ERROR:
			str = "No error";
			break;
		case ESC_AL_STATUS_UNSPECIFIED:
			str = "Unspecified error";
			break;
		case ESC_AL_STATUS_INVALID_REQ:
			str = "Invalid requested state change";
			break;
		case ESC_AL_STATUS_UNKNOWN_REQ:
			str = "Unknown requested state";
			break;
		case ESC_AL_STATUS_BOOTSTRAP_UNSUP:
			str = "Bootstrap not supported";
			break;
		case ESC_AL_STATUS_NO_VALID_FMW:
			str = "No valid firmware";
			break;
		case ESC_AL_STATUS_INVALID_MBOX1:
			str = "Invalid mailbox configuration";
			break;
		case ESC_AL_STATUS_INVALID_MBOX2:
			str = "Invalid mailbox configuration";
			break;
		case ESC_AL_STATUS_INVALID_SM:
			str = "Invalid sync manager configuration";
			break;
		case ESC_AL_STATUS_NO_VALID_INPUT:
			str = "No valid inputs available";
			break;
		case ESC_AL_STATUS_NO_VALID_OUTPUT:
			str = "No valid outputs";
			break;
		case ESC_AL_STATUS_SYNC_ERROR:
			str = "Synchronization error";
			break;
		case ESC_AL_STATUS_SM_WATCHDOG:
			str = "Sync manager watchdog";
			break;
		case ESC_AL_STATUS_INVALID_SM_TYPES:
			str = "Invalid Sync Manager Types";
			break;
		case ESC_AL_STATUS_INVALID_OUT_CONF:
			str = "Invalid Output Configuration";
			break;
		case ESC_AL_STATUS_INVALID_IN_CONF:
			str = "Invalid Input Configuration";
			break;
		case ESC_AL_STATUS_INVALID_WD_CONF:
			str = "Invalid Watchdog Configuration";
			break;
		case ESC_AL_STATUS_SLAVE_COLD:
			str = "Slave needs cold start";
			break;
		case ESC_AL_STATUS_SLAVE_INIT:
			str = "Slave needs INIT";
			break;
		case ESC_AL_STATUS_SLAVE_PREOP:
			str = "Slave needs PREOP";
			break;
		case ESC_AL_STATUS_SLAVE_SAFEOP:
			str = "Slave needs SAFEOP";
			break;
		case ESC_AL_STATUS_INVALID_DC_S_CONF:
			str = "Invalid DC SYNCH Configuration";
			break;
		case ESC_AL_STATUS_INVALID_DC_L_CONF:
			str = "Invalid DC Latch Configuration";
			break;
		case ESC_AL_STATUS_PLL_ERROR:
			str = "PLL Error";
			break;
		case ESC_AL_STATUS_INVALID_DC_IO:
			str = "Invalid DC IO Error";
			break;
		case ESC_AL_STATUS_INVALID_DC_TO:
			str = "Invalid DC Timeout Error";
			break;
		case ESC_AL_STATUS_DC_SYNC0_CYCLE:
			str = "Invalid DC SYNC0 Cycle Time";
			break;
		case ESC_AL_STATUS_DC_SYNC1_CYCLE:
			str = "Invalid DC SYNC1 Cycle Time";
			break;
		case ESC_AL_STATUS_MBOX_EOE:
			str = "MBX_EOE";
			break;
		case ESC_AL_STATUS_MBOX_COE:
			str = "MBX_COE";
			break;
		case ESC_AL_STATUS_MBOX_FOE:
			str = "MBX_FOE";
			break;
		case ESC_AL_STATUS_MBOX_SOE:
			str = "MBX_SOE";
			break;
		case ESC_AL_STATUS_MBOX_VOE:
			str = "MBX_VOE";
			break;
		default:
			str = "unknown";
			break;
	}
	return str;
};

int esc_al_error_ack(struct ec_dgram_addr *addr);
enum esc_al_status esc_al_status_code_get(struct ec_dgram_addr *addr);

int esc_pdo_tx_set(struct ec_dgram_addr *addr,
    struct esc_syncmanager_info *sm, unsigned char **data,
    bool watchdog);
int esc_pdo_rx_set(struct ec_dgram_addr *addr,
    struct esc_syncmanager_info *sm, unsigned char **data);


#endif /* _INCLUDE_EC_ESC_H_ */
